include Summation.inc
include Seh-2.inc
;**********************CODE SEGMENT********************************
.code 
start:; START PROG HERE
		invoke	InitCommonControls
		invoke	GetModuleHandle,NULL
		mov	hInstance,eax
		invoke	DialogBoxParam,hInstance,IDD_SUMMATION,NULL,offset DlgProc,0
		invoke	ExitProcess,NULL
;******************************************************************
;An example of calling Sum for uNum = 0 through 9
; uNum: 0 1 2 3  4  5  6  7  8  9 ...
; Sum:  0 1 3 6 10 15 21 28 36 45 ...
Sum  proc uNum:dword
	;Call Sum recursively.
	mov eax,uNum
	.if eax!=0
		dec uNum
		invoke Sum,uNum		
		add eax,uNum
		inc eax
	.else
	.endif
    ret
Sum endp
;************FilterFunc PROC**************************************
FilterFunc  proc dwExceptionCode:dword
	.if dwExceptionCode == STATUS_STACK_OVERFLOW
		mov eax,EXCEPTION_EXECUTE_HANDLER
	.else
		mov eax, EXCEPTION_CONTINUE_SEARCH
	.endif
     ret
FilterFunc endp
;********************************************************************
;The separate thread that is responsible for calculating the sum. 
; I use a separate thread for the following reasons:
;    1. A separate thread gets its own 1 MB of stack space.
;    2. A thread can be notified of a stack overflow only once.
;    3. The stack's storage is freed when the thread exits.
SumThreadFunc  proc pvParam:dword
;local uSumNum:dword
local uSum:dword		
	InstSehFrame OFFSET SAFE_PLACE
	;The parameter pvParam, contains the number of integers to sum.	
	;uSum contains the summation of the numbers from 0 through uSumNum. 
    ;If the sum cannot be calculated, a sum of UINT_MAX is returned.
	;mov eax, pvParam
	;mov uSumNum,eax
	;To catch the stack overflow exception, we must 
    ; execute the Sum function while inside an SEH block.	
	invoke Sum,pvParam
	;invoke Sum,uSumNum
	;The thread's exit code is the sum of the first uSumNum 
    ; numbers, or UINT_MAX if a stack overflow occurred.	 
	
	KillSehFrame	
	ret
SAFE_PLACE:	
	;************************************
	;If we get in here, it's because we have trapped a stack overflow. 
    ; We can now do whatever is necessary to gracefully continue 
	;execution
	KillSehFrame			
	mov eax,UINT_MAX		
	ret
SumThreadFunc endp

;***********DIALOG MESSAGE PROCEDURE*****************************
DlgProc	proc hWnd:DWORD,uMsg:DWORD,wParam:DWORD,lParam:DWORD
local uSum:dword
local dwThreadId:dword
local hThread:dword
		mov	eax,uMsg
		.if	eax == WM_CLOSE
			invoke	EndDialog,hWnd,NULL
		.elseif	eax == WM_INITDIALOG
			invoke LoadIcon,hInstance,IDI_SUMMATION
			push eax
			invoke SendMessage,hWnd,WM_SETICON,ICON_SMALL,eax		
			pop eax
			invoke SendMessage,hWnd,WM_SETICON,ICON_BIG,eax
			
		.elseif	eax == WM_COMMAND
			mov	eax,wParam
			.if eax == IDCANCEL
				invoke	EndDialog,hWnd,NULL
			.elseif eax==IDC_CALC
				;Get the number of integers the user wants to sum.
				invoke GetDlgItemInt,hWnd, IDC_SUMNUM, NULL, FALSE
				mov uSum,eax
				;Create a thread (with its own stack) that is
                ; responsible for performing the summation.
				invoke CreateThread,NULL, 0,addr SumThreadFunc,uSum, 0,addr dwThreadId
				mov hThread,eax
				;Wait for the thread to terminate.
				invoke WaitForSingleObject,hThread, INFINITE
				;The thread's exit code is the resulting summation.
				invoke GetExitCodeThread,hThread,addr uSum
				;Allow the system to destroy the thread kernel object
				invoke CloseHandle,hThread
				; Update the dialog box to show the result.
				.if uSum == UINT_MAX
					;If result is UINT_MAX, a stack overflow occurred.
					invoke SetDlgItemText,hWnd,IDC_ANSWER,SADD("Error")
					invoke MessageBox,hWnd,SADD("The number is too big, please enter a smaller number"),SADD("Error"),MB_OK
				.else
					;The sum was calculated successfully; 
					invoke SetDlgItemInt,hWnd, IDC_ANSWER, uSum, FALSE
				.endif

			.endif
		.else
			mov	eax,FALSE
			ret
		.endif		   
		mov	eax,TRUE
		ret
		
DlgProc	endp
;********************************************************************

end start